==> elkcode_9.2.5.orig.tar.gz: elk-9.2.5/src/ksinvert.f90 <==

! Copyright (C) 2018 P. Elliott, J. K. Dewhurst, S. Sharma and E. K. U. Gross.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

subroutine ksinvert
use modmain
use modmpi
implicit none
! local variables
integer idm,it
real(8) resp
! allocatable arrays
real(8), allocatable :: rhomt_(:,:),rhoir_(:)
real(8), allocatable :: magmt_(:,:,:),magir_(:,:)
real(8), allocatable :: rfmt(:,:),rfir(:)
! copy the existing density and magnetisation
allocate(rhomt_(npmtmax,natmtot),rhoir_(ngtot))
call rfcopy(rhomt,rhoir,rhomt_,rhoir_)
if (spinpol) then
  allocate(magmt_(npmtmax,natmtot,ndmag),magir_(ngtot,ndmag))
  do idm=1,ndmag
    call rfcopy(magmt(:,:,idm),magir(:,idm),magmt_(:,:,idm),magir_(:,idm))
  end do
end if
allocate(rfmt(npmtmax,natmtot),rfir(ngtot))
! zero the exchange-correlation potential and magnetic field
vxcmt(:,:)=0.d0
vxcir(:)=0.d0
if (spinpol) then
  bxcmt(:,:,:)=0.d0
  bxcir(:,:)=0.d0
end if
if (mp_mpi) then
  write(*,'("Info(ksinvert): inverting the Kohn-Sham equations")')
end if
do it=1,maxitksi
  if (mp_mpi.and.(mod(it,10) == 0)) then
    write(*,'("Info(ksinvert): done ",I4," iterations of ",I4)') it,maxitksi
  end if
  call hmlrad
  call genevfsv
  call occupy
  call rhomag
! determine the residual and add it to the exchange-correlation potential
  call residual
! update the step size
  if (it >= 2) then
! check residual against previous value
    if (resksi < resp) then
      tauksi=tauksi+tau0ksi
    else
      tauksi=tau0ksi
    end if
  end if
! store previous residual
  resp=resksi
! set the constant part of V_xc equal to zero
  call rfint0(0.d0,vxcmt,vxcir)
! add the external and Hartree potentials
  call potks(.false.)
  call genvsig
end do
deallocate(rhomt_,rhoir_,rfmt,rfir)
if (spinpol) deallocate(magmt_,magir_)
! synchronise MPI processes
call mpi_barrier(mpicom,ierror)
return

contains

subroutine residual
implicit none
! local variables
integer is,ias,np
! external functions
real(8), external :: rfinp
do ias=1,natmtot
  is=idxis(ias)
  np=npmt(is)
  rfmt(1:np,ias)=rhomt(1:np,ias)-rhomt_(1:np,ias)
  vxcmt(1:np,ias)=vxcmt(1:np,ias)+tauksi*rfmt(1:np,ias)
end do
rfir(:)=rhoir(:)-rhoir_(:)
vxcir(:)=vxcir(:)+tauksi*rfir(:)
resksi=rfinp(rfmt,rfir,rfmt,rfir)
if (spinpol) then
  do idm=1,ndmag
    do ias=1,natmtot
      is=idxis(ias)
      np=npmt(is)
      rfmt(1:np,ias)=magmt(1:np,ias,idm)-magmt_(1:np,ias,idm)
      bxcmt(1:np,ias,idm)=bxcmt(1:np,ias,idm)+tauksi*rfmt(1:np,ias)
    end do
    rfir(:)=magir(:,idm)-magir_(:,idm)
    bxcir(:,idm)=bxcir(:,idm)+tauksi*rfir(:)
    resksi=resksi+rfinp(rfmt,rfir,rfmt,rfir)
  end do
end if
resksi=sqrt(resksi)/omega
end subroutine

end subroutine

==> elkcode_9.2.12.orig.tar.gz: elk-9.2.12/src/ksinvert.f90 <==

! Copyright (C) 2018 P. Elliott, J. K. Dewhurst, S. Sharma and E. K. U. Gross.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

subroutine ksinvert
use modmain
use modmpi
implicit none
! local variables
integer idm,it
! allocatable arrays
real(8), allocatable :: rhomt_(:,:),rhoir_(:)
real(8), allocatable :: magmt_(:,:,:),magir_(:,:)
real(8), allocatable :: rfmt(:,:),rfir(:)
! copy the existing density and magnetisation
allocate(rhomt_(npmtmax,natmtot),rhoir_(ngtot))
call rfcopy(rhomt,rhoir,rhomt_,rhoir_)
if (spinpol) then
  allocate(magmt_(npmtmax,natmtot,ndmag),magir_(ngtot,ndmag))
  do idm=1,ndmag
    call rfcopy(magmt(:,:,idm),magir(:,idm),magmt_(:,:,idm),magir_(:,idm))
  end do
end if
allocate(rfmt(npmtmax,natmtot),rfir(ngtot))
if (mp_mpi) then
  write(*,'("Info(ksinvert): inverting the Kohn-Sham equations")')
end if
do it=1,maxitksi
  if (mp_mpi.and.(mod(it,10) == 0)) then
    write(*,'("Info(ksinvert): done ",I4," iterations of ",I4)') it,maxitksi
  end if
  call gencore
  call hmlrad
  call genevfsv
  call occupy
  call rhomag
! determine the residual and add it to the exchange-correlation potential
  call residual
! set the constant part of V_xc equal to zero
  call rfint0(0.d0,vxcmt,vxcir)
! add the external and Hartree potentials
  call potks(.false.)
  call genvsig
end do
deallocate(rhomt_,rhoir_,rfmt,rfir)
if (spinpol) deallocate(magmt_,magir_)
! synchronise MPI processes
call mpi_barrier(mpicom,ierror)
return

contains

subroutine residual
implicit none
! local variables
integer is,ias,np
! external functions
real(8), external :: rfinp
do ias=1,natmtot
  is=idxis(ias)
  np=npmt(is)
  rfmt(1:np,ias)=rhomt(1:np,ias)-rhomt_(1:np,ias)
  vxcmt(1:np,ias)=vxcmt(1:np,ias)+tauksi*rfmt(1:np,ias)
end do
rfir(:)=rhoir(:)-rhoir_(:)
vxcir(:)=vxcir(:)+tauksi*rfir(:)
resksi=rfinp(rfmt,rfir,rfmt,rfir)
if (spinpol) then
  do idm=1,ndmag
    do ias=1,natmtot
      is=idxis(ias)
      np=npmt(is)
      rfmt(1:np,ias)=magmt(1:np,ias,idm)-magmt_(1:np,ias,idm)
      bxcmt(1:np,ias,idm)=bxcmt(1:np,ias,idm)+tauksi*rfmt(1:np,ias)
    end do
    rfir(:)=magir(:,idm)-magir_(:,idm)
    bxcir(:,idm)=bxcir(:,idm)+tauksi*rfir(:)
    resksi=resksi+rfinp(rfmt,rfir,rfmt,rfir)
  end do
end if
resksi=sqrt(resksi)/omega
end subroutine

end subroutine
